@@ -36,13 +36,21 @@ public:
     Q_SLOT void updateCommand();
 
     Q_SLOT void invokeCommand(const QModelIndex &idx);
+    // Run a command from the currently selected Tree element
 
-    Q_SLOT void triggerEdit();
+    Q_SLOT void runCommand();
+    // Run the command from the Text Area
+
+    Q_SLOT void indexSelected(const QModelIndex &idx);
 
     Q_SLOT void triggerRename();
 
     Q_SLOT void triggerDelete();
 
+    Q_SLOT void createMenu(const QPoint &pos);
+
+    Q_SLOT void runShellCheck();
+
     void setModel(QuickCommandsModel *model);
     void setCurrentController(Konsole::SessionController *controller);
 
